-------------------------------------------------------------------------------
--|> Crosshair
-- Places a crosshair on a nameplate.
-- Filtered for player targets that are hostile player with support for icons, text etc attachments
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local IsItemInRange = IsItemInRange
local C_NamePlate = C_NamePlate
local UnitIsPlayer = UnitIsPlayer
local UnitIsEnemy = UnitIsEnemy
local UnitSelectionColor = UnitSelectionColor
local UnitClass = UnitClass
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local IsFlying = IsFlying
local IsUsableItem = IsUsableItem
local UnitIsUnit = UnitIsUnit
local GetTime = GetTime
local C_Timer_After = C_Timer.After

-------------------------------------------------------------------------------
--|> LIBS
-------------------------------------------------------------------------------
local RC = LibStub("LibRangeCheck-2.0")

-------------------------------------------------------------------------------
--|> LOCALS
-------------------------------------------------------------------------------
local _
local TargetNameplate = nil
local lineAlpha = 0.4

weizPVP.ReplaceTargetedNameplate = true
weizPVP.Display.Width = GetScreenWidth()
weizPVP.Display.Height = GetScreenHeight()
weizPVP.Display.xFactor = (768 / weizPVP.Display.Width) * (weizPVP.Display.Width / weizPVP.Display.Height)
weizPVP.Display.yFactor = 768 / weizPVP.Display.Height
weizPVP.Display.EffectiveScale = UIParent:GetEffectiveScale()

-------------------------------------------------------------------------------
--|> CREATE CORE FRAME
-------------------------------------------------------------------------------
--> CROSSHAIR FRAME
-------------------------------------------------------
weizPVP.CrosshairFrame = CreateFrame("frame", "weizPVP:CrosshairFrame", WorldFrame)
weizPVP.CrosshairFrame:SetFrameStrata("BACKGROUND")
weizPVP.CrosshairFrame:SetFrameLevel(111)
weizPVP.CrosshairFrame:SetPoint("CENTER")
weizPVP.CrosshairFrame:SetSize(64, 64)
weizPVP.CrosshairFrame:Hide()
if weizPVP.Options.Crosshair.Enabled == true then
	weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
end

--> CROSSHAIR ANIMATION: FADE IN
-------------------------------------------------------
local CrosshairFadeInGroup = weizPVP.CrosshairFrame:CreateAnimationGroup()
CrosshairFadeInGroup:SetToFinalAlpha(true)
local CrosshairFadeIn = CrosshairFadeInGroup:CreateAnimation("Alpha")
CrosshairFadeIn:SetSmoothing("OUT")
CrosshairFadeIn:SetFromAlpha(0)
CrosshairFadeIn:SetToAlpha(1)
CrosshairFadeIn:SetDuration(0.1)
CrosshairFadeInGroup:SetScript(
	"OnPlay",
	function(self)
		weizPVP.CrosshairFrame:Show()
	end
)

--> CROSSHAIR ANIMATION: FADE OUT
-------------------------------------------------------
local CrosshairFadeOutGroup = weizPVP.CrosshairFrame:CreateAnimationGroup()
CrosshairFadeOutGroup:SetToFinalAlpha(true)
local CrosshairFadeOut = CrosshairFadeOutGroup:CreateAnimation("Alpha")
CrosshairFadeOut:SetSmoothing("IN")
CrosshairFadeOut:SetFromAlpha(1)
CrosshairFadeOut:SetToAlpha(0)
CrosshairFadeOut:SetDuration(0.2)
CrosshairFadeOutGroup:SetScript(
	"OnFinished",
	function(self)
		weizPVP.CrosshairFrame:Hide()
	end
)

-------------------------------------------------------------------------------
--|> CREATE TEXTURES
-------------------------------------------------------------------------------

--=> TARGET'S  BASE CIRCLE, RECTILE, and BACKGROUND
---------------------------------------------------------------------

--> Center Ring
-------------------------------------------------------
local CenterRing = weizPVP.CrosshairFrame:CreateTexture(nil, "BACKGROUND")
CenterRing:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/media/classring.tga]])
CenterRing:SetPoint("CENTER", weizPVP.CrosshairFrame, "CENTER")
CenterRing:SetWidth(128)
CenterRing:SetHeight(128)
CenterRing:SetScale(weizPVP.Display.EffectiveScale)
CenterRing:SetAlpha(1)
CenterRing:SetDrawLayer("ARTWORK", 4)
CenterRing:SetBlendMode("ADD")

--> Center Ring: Fade Group
-----------------------------------------------------
local CenterRingFadeGroup = CenterRing:CreateAnimationGroup()
CenterRingFadeGroup:SetToFinalAlpha(true)
local CenterRingFade = CenterRingFadeGroup:CreateAnimation("Alpha")
CenterRingFade:SetFromAlpha(0)
CenterRingFade:SetToAlpha(1)
CenterRingFade:SetDuration(0.4)

--> Ring Shine
-------------------------------------------------------
local RingShine = weizPVP.CrosshairFrame:CreateTexture(nil, "BACKGROUND")
RingShine:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/media/circle_shine.tga]])
RingShine:SetPoint("CENTER", weizPVP.CrosshairFrame, "CENTER")
RingShine:SetVertexColor(1, 1, 1)
RingShine:SetWidth(128)
RingShine:SetHeight(128)
RingShine:SetDrawLayer("BACKGROUND", 3)
RingShine:SetScale(weizPVP.Display.EffectiveScale)
RingShine:SetAlpha(1)
RingShine:SetBlendMode("ADD")

--> Class Ring: Fade Group
-----------------------------------------------------
local RingShineFadeGroup = RingShine:CreateAnimationGroup()
RingShineFadeGroup:SetToFinalAlpha(true)
local RingShineFade = RingShineFadeGroup:CreateAnimation("Alpha")
RingShineFade:SetFromAlpha(0)
RingShineFade:SetToAlpha(1)
RingShineFade:SetDuration(0.4)

--> Circle Shine Animation: Rotate
-------------------------------------------------------
local RingShine360Group = RingShine:CreateAnimationGroup()
local RingShineRotation = RingShine360Group:CreateAnimation("Rotation")
RingShineRotation:SetDegrees(-360)
RingShineRotation:SetDuration(12)
RingShine360Group:SetLooping("REPEAT")
RingShine360Group:Play()

--> Background
-------------------------------------------------------
local CrosshairBackgroundShadow = weizPVP.CrosshairFrame:CreateTexture(nil, "BACKGROUND")
CrosshairBackgroundShadow:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/media/classring_bg.tga]])
CrosshairBackgroundShadow:SetPoint("CENTER", weizPVP.CrosshairFrame, "CENTER")
CrosshairBackgroundShadow:SetWidth(128)
CrosshairBackgroundShadow:SetHeight(128)
CrosshairBackgroundShadow:SetDrawLayer("BACKGROUND", -6)
CrosshairBackgroundShadow:SetVertexColor(0, 0, 0, 1)
CrosshairBackgroundShadow:SetScale(weizPVP.Display.EffectiveScale)
CrosshairBackgroundShadow:SetAlpha(1)
CrosshairBackgroundShadow:SetBlendMode("BLEND")

-------------------------------------------------------------------------------
--|>|>  Four Arrows  <|<|--
-------------------------------------------------------------------------------
--> Four Arrows Texture
-------------------------------------------------------
local FourArrows = weizPVP.CrosshairFrame:CreateTexture(nil, "BACKGROUND")
FourArrows:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/media/4Arrows.tga]])
FourArrows:SetPoint("CENTER", weizPVP.CrosshairFrame, "CENTER")
FourArrows:SetWidth(96)
FourArrows:SetHeight(96)
FourArrows:SetDrawLayer("ARTWORK", 1)
FourArrows:SetVertexColor(1, 0, 0)
FourArrows:SetScale(1)
FourArrows:SetBlendMode("ADD")
FourArrows:SetRotation(0.7853975)

--> FourArrows: Transition to Target Animation
-------------------------------------------------------
-->> Scale In
local FourArrowsAniGTarget1 = FourArrows:CreateAnimationGroup()
local ArrowScaleIN = FourArrowsAniGTarget1:CreateAnimation("Scale")
ArrowScaleIN:SetDuration(0.4)
ArrowScaleIN:SetFromScale(3, 3)
ArrowScaleIN:SetToScale(1, 1)
ArrowScaleIN:SetSmoothing("OUT")
-->> Fade In
local FourArrowsAniGTarget2 = FourArrows:CreateAnimationGroup()
local ArrowAlphaIN = FourArrowsAniGTarget2:CreateAnimation("Alpha")
ArrowAlphaIN:SetDuration(0.4)
ArrowAlphaIN:SetFromAlpha(0)
ArrowAlphaIN:SetToAlpha(1)

-------------------------------------------------------------------------------
--|> CREATE LINES
-------------------------------------------------------------------------------
local lineOffset = 2
-->> LEFT
-------------------------------------------------------
local leftLine = weizPVP.CrosshairFrame:CreateLine("BACKGROUND")
leftLine:SetTexture([[interface/addons/weizPVP/Textures/white16x16.tga]])
leftLine:SetThickness(2)
leftLine:SetStartPoint("LEFT", weizPVP.CrosshairFrame, UIParent:GetLeft() - weizPVP.CrosshairFrame:GetLeft(), 0)
leftLine:SetEndPoint("LEFT", weizPVP.CrosshairFrame, lineOffset, 0)
leftLine:SetVertexColor(1, 1, 1)
leftLine:SetAlpha(lineAlpha)
leftLine:SetBlendMode("ADD")
-->> RIGHT
-------------------------------------------------------
local rightLine = weizPVP.CrosshairFrame:CreateLine("BACKGROUND")
rightLine:SetTexture([[interface/addons/weizPVP/Textures/white16x16.tga]])
rightLine:SetThickness(2)
rightLine:SetStartPoint("RIGHT", weizPVP.CrosshairFrame, UIParent:GetRight() - weizPVP.CrosshairFrame:GetRight(), 0)
rightLine:SetEndPoint("RIGHT", weizPVP.CrosshairFrame, -1 * lineOffset, 0)
rightLine:SetVertexColor(1, 1, 1)
rightLine:SetAlpha(lineAlpha)
rightLine:SetBlendMode("ADD")
-->> TOP
-------------------------------------------------------
local topLine = weizPVP.CrosshairFrame:CreateLine("BACKGROUND")
topLine:SetTexture([[interface/addons/weizPVP/Textures/white16x16.tga]])
topLine:SetThickness(2)
topLine:SetStartPoint("TOP", weizPVP.CrosshairFrame, 0, UIParent:GetTop() - weizPVP.CrosshairFrame:GetTop())
topLine:SetEndPoint("TOP", weizPVP.CrosshairFrame, 0, -1 * lineOffset)
topLine:SetVertexColor(1, 1, 1)
topLine:SetAlpha(lineAlpha)
topLine:SetBlendMode("ADD")
-->> BOTTOM
-------------------------------------------------------
local bottomLine = weizPVP.CrosshairFrame:CreateLine("BACKGROUND")
bottomLine:SetTexture([[interface/addons/weizPVP/Textures/white16x16.tga]])
bottomLine:SetThickness(2)
bottomLine:SetStartPoint("BOTTOM", weizPVP.CrosshairFrame, 0, UIParent:GetBottom() - weizPVP.CrosshairFrame:GetBottom())
bottomLine:SetEndPoint("BOTTOM", weizPVP.CrosshairFrame, 0, lineOffset)
bottomLine:SetVertexColor(1, 1, 1)
bottomLine:SetAlpha(lineAlpha)
bottomLine:SetBlendMode("ADD")

-------------------------------------------------------------------------------
--|> CREATE ADDITIONAL INDICATORS
--** Additional UI elements connected to Crosshair can be placed here.
-------------------------------------------------------------------------------

--> Target NameText
---------------------------------------------------------------------
local NameText = weizPVP.CrosshairFrame:CreateFontString(nil, "OVERLAY", nil, 1)
NameText:SetFont([[Interface\Addons\weizPVP\Fonts\RobotoCondensed-BoldItalic.ttf]], 16, "NONE")
NameText:SetHeight(16)
NameText:SetPoint("BOTTOMLEFT", rightLine, "TOPLEFT", weizPVP.ScaleCoords(16, 4))
NameText:SetJustifyH("LEFT")
NameText:SetText("")
NameText:SetTextColor(1, 1, 1)
NameText:SetShadowColor(0, 0, 0, 1)
NameText:SetShadowOffset(-1, 1)
NameText:SetAlpha(1)
if not weizPVP.ReplaceTargetedNameplate then
	NameText:Hide()
end

--> Target Name Text Background Text
-------------------------------------------------------
local NameTextBG = weizPVP.CrosshairFrame:CreateFontString(nil, "BACKGROUND", nil, 1)
NameTextBG:SetFont([[Interface\Addons\weizPVP\Fonts\RobotoCondensed-BoldItalic.ttf]], 16, "OUTLINE")
NameTextBG:SetPoint("CENTER", NameText, "CENTER", weizPVP.ScaleCoords(2, -1))
NameTextBG:SetDrawLayer("BACKGROUND", 1)
NameTextBG:SetHeight(NameText:GetHeight())
NameTextBG:SetText(NameText:GetText())
NameTextBG:SetJustifyH(NameText:GetJustifyH())
NameTextBG:SetTextColor(0, 0, 0)
NameTextBG:SetShadowColor(0, 0, 0, 1)
NameTextBG:SetShadowOffset(1, -1)
NameTextBG:SetAlpha(1)
if not weizPVP.ReplaceTargetedNameplate then
	NameTextBG:Hide()
end

--> Target Name: BG GRAD
-------------------------------------------------------
local NameTextGradBG = weizPVP.CrosshairFrame:CreateTexture(nil, "BACKGROUND")
NameTextGradBG:SetHeight(12)
NameTextGradBG:SetPoint("LEFT", rightLine, "LEFT", 0, 0)
NameTextGradBG:SetPoint("RIGHT", NameText, "RIGHT", weizPVP.ScaleCoords(12, 0))
NameTextGradBG:SetPoint("BOTTOM", rightLine, "TOP", weizPVP.ScaleCoords(0, 1))
NameTextGradBG:SetTexture(1, 0, 0, 1)
NameTextGradBG:SetGradientAlpha("Vertical", 0, 0, 0, 0.4, 0, 0, 0, 0)

--> Target Name: BG GRAD END CORNER
-------------------------------------------------------
local NameTextGradBGCorner = weizPVP.CrosshairFrame:CreateTexture(nil, "BACKGROUND")
NameTextGradBGCorner:SetHeight(NameTextGradBG:GetHeight())
NameTextGradBGCorner:SetWidth(NameTextGradBG:GetHeight())
NameTextGradBGCorner:SetPoint("TOPLEFT", NameTextGradBG, "TOPRIGHT", weizPVP.ScaleCoords(0, 0))
NameTextGradBGCorner:SetTexture([[interface/addons/weizPVP/Textures/cornerGrad.tga]])
NameTextGradBGCorner:SetVertexColor(0, 0, 0, 0.4)
NameTextGradBGCorner:SetAlpha(0.4)
NameTextGradBGCorner:SetRotation(1.570795)
NameTextGradBGCorner:SetBlendMode("BLEND")

--> Target: Guild Text
---------------------------------------------------------------------
local GuildText = weizPVP.CrosshairFrame:CreateFontString(nil, "ARTWORK", nil, 1)
GuildText:SetFont([[Interface\Addons\weizPVP\Fonts\Accidental-Presidency-Italic.ttf]], 12, "NONE")
GuildText:SetHeight(12)
GuildText:SetPoint("TOPLEFT", NameText, "BOTTOMLEFT", weizPVP.ScaleCoords(0, -4))
GuildText:SetJustifyH("LEFT")
GuildText:SetText("")
GuildText:SetTextColor(.9, .9, .9)
GuildText:SetShadowColor(0, 0, 0, 1)
GuildText:SetShadowOffset(-1, 1)
GuildText:SetAlpha(1)
if not weizPVP.ReplaceTargetedNameplate then
	GuildText:Hide()
end

--> Guild Text: BG
-------------------------------------------------------
local GuildTextBG = weizPVP.CrosshairFrame:CreateFontString(nil, "BACKGROUND", nil, 1)
GuildTextBG:SetFont([[Interface\Addons\weizPVP\Fonts\Accidental-Presidency-Italic.ttf]], GuildText:GetHeight(), "NONE")
GuildTextBG:SetPoint("CENTER", GuildText, "CENTER", weizPVP.ScaleCoords(1, -1))
GuildTextBG:SetDrawLayer("BACKGROUND", 6)
GuildTextBG:SetHeight(GuildText:GetHeight())
GuildTextBG:SetText(GuildText:GetText())
GuildTextBG:SetJustifyH(GuildText:GetJustifyH())
GuildTextBG:SetTextColor(0.1, 0.1, 0.1)
GuildTextBG:SetShadowColor(0, 0, 0, 1)
GuildTextBG:SetShadowOffset(1, -1)
GuildTextBG:SetAlpha(1)
if not weizPVP.ReplaceTargetedNameplate then
	GuildTextBG:Hide()
end

--> Guild Text: BG GRAD
-------------------------------------------------------
local GuildTextGradBG = weizPVP.CrosshairFrame:CreateTexture(nil, "BACKGROUND")
GuildTextGradBG:SetHeight(14)
GuildTextGradBG:SetPoint("LEFT", rightLine, "LEFT", 0, 0)
GuildTextGradBG:SetPoint("RIGHT", GuildText, "RIGHT", weizPVP.ScaleCoords(12, 0))
GuildTextGradBG:SetPoint("TOP", rightLine, "TOP", weizPVP.ScaleCoords(0, -1))
GuildTextGradBG:SetTexture(1, 0, 0, 1)
GuildTextGradBG:SetGradientAlpha("Vertical", 0, 0, 0, 0, 0, 0, 0, 0.4)
GuildTextGradBG:SetDrawLayer("BACKGROUND", -7)

--> Guild Text: BG GRAD END CORNER
-------------------------------------------------------
local GuildTextGradBGCorner = weizPVP.CrosshairFrame:CreateTexture(nil, "BACKGROUND")
GuildTextGradBGCorner:SetHeight(GuildTextGradBG:GetHeight())
GuildTextGradBGCorner:SetWidth(GuildTextGradBG:GetHeight())
GuildTextGradBGCorner:SetPoint("TOPLEFT", GuildTextGradBG, "TOPRIGHT", weizPVP.ScaleCoords(0, 0))
GuildTextGradBGCorner:SetTexture([[interface/addons/weizPVP/Textures/cornerGrad.tga]])
GuildTextGradBGCorner:SetVertexColor(0, 0, 0, 0.4)
GuildTextGradBGCorner:SetAlpha(0.4)
GuildTextGradBGCorner:SetBlendMode("BLEND")
GuildTextGradBGCorner:SetDrawLayer("BACKGROUND", -7)

--> Net-O-Matic Icon
---------------------------------------------------------------------
local iconNetOMatic = weizPVP.CrosshairFrame:CreateTexture(nil, "ARTWORK")
iconNetOMatic:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/media/netomatic.tga]])
iconNetOMatic:SetSize(24, 24)
iconNetOMatic:SetPoint("BOTTOMLEFT", rightLine, "TOPLEFT", 0, 12)
iconNetOMatic:SetAlpha(0)

-------------------------------------------------------------------------------
--|> PLUGIN CORE FUNCTIONS
-------------------------------------------------------------------------------

--> ENABLE CROSSHAIR
-------------------------------------------------------
function weizPVP.EnableCrosshair()
	--> Register Events
	weizPVP.CrosshairFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
	weizPVP.CrosshairFrame:RegisterEvent("NAME_PLATE_UNIT_REMOVED")
	weizPVP.CrosshairFrame:RegisterEvent("NAME_PLATE_UNIT_ADDED")
	weizPVP.CrosshairFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
	weizPVP.Options.Crosshair.Enabled = true
	--> Check current target
	weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
end

--> DISABLE CROSSHAIR
-------------------------------------------------------
function weizPVP:DisableCrosshair()
	--> Clear plate and fade out
	TargetNameplate = nil
	CrosshairFadeOutGroup:Play()

	--> Unregister Events
	weizPVP.CrosshairFrame:UnregisterEvent("PLAYER_TARGET_CHANGED")
	weizPVP.CrosshairFrame:UnregisterEvent("NAME_PLATE_UNIT_REMOVED")
	weizPVP.CrosshairFrame:UnregisterEvent("NAME_PLATE_UNIT_ADDED")
	weizPVP.CrosshairFrame:UnregisterEvent("PLAYER_ENTERING_WORLD")
	weizPVP.Options.Crosshair.Enabled = false
end

-------------------------------------------------------------------------------
--|> UI FUNCTIONS
-------------------------------------------------------------------------------

--=> Set Color of lines and circle
-------------------------------------------------------
local CurrentClassColor = {1, 1, 1}
local function SetColor(r, g, b)
	CurrentClassColor = {r, g, b}
	--> Rotating Circles
	NameText:SetTextColor(weizPVP:AdjustRGBTint({r, g, b}, 0.3))
	NameTextBG:SetTextColor(weizPVP:AdjustRGBTint({r, g, b}, -0.6))

	--> Lines
	topLine:SetVertexColor(weizPVP:AdjustRGBTint({r, g, b}, -0.1))
	bottomLine:SetVertexColor(weizPVP:AdjustRGBTint({r, g, b}, -0.1))
	leftLine:SetVertexColor(weizPVP:AdjustRGBTint({r, g, b}, -0.1))
	rightLine:SetVertexColor(weizPVP:AdjustRGBTint({r, g, b}, -0.1))
end

--=> Get Nameplate
-------------------------------------------------------
local Moving = GetTime()
local x
local y
local fScale
local class
local classColor
local r, g, b
function weizPVP:GetNameplate(plate)
	--> Set on same level as nameplate
	weizPVP.CrosshairFrame:SetFrameLevel(tonumber(plate:GetFrameLevel()) - 1)
	x, y = nil, nil
	if not weizPVP.CrosshairFrame:IsShown() then
		x, y = plate:GetCenter()
		if x and y then
			x, y = x * weizPVP.CrosshairFrame:GetEffectiveScale(), y * weizPVP.CrosshairFrame:GetEffectiveScale()
			fScale = weizPVP.CrosshairFrame:GetScale()
			x, y = x / fScale, y / fScale
			weizPVP.CrosshairFrame:SetPoint("CENTER", nil, "BOTTOMLEFT", weizPVP.ScaleCoords(x, y))
		end
		if not CrosshairFadeInGroup:IsPlaying() then
			CrosshairFadeOutGroup:Stop()
			CrosshairFadeInGroup:Play()
		end
	else
		weizPVP.CrosshairFrame:Show()
	end
	-- TargetNameplate = plate
	if FourArrowsAniGTarget1:IsPlaying() then
		FourArrowsAniGTarget1:Stop()
	end
	if FourArrowsAniGTarget2:IsPlaying() then
		FourArrowsAniGTarget2:Stop()
	end
	FourArrowsAniGTarget1:Play()
	FourArrowsAniGTarget2:Play()

	--> Set Class Colors
	r, g, b = UnitSelectionColor("target")
	if UnitIsPlayer("target") then
		class = select(2, UnitClass("target"))
		classColor = RAID_CLASS_COLORS[class]
		if classColor then
			r, g, b = classColor.r, classColor.g, classColor.b
		end
	end
	--> APPLY COLORS
	SetColor(r, g, b)
	Moving = GetTime()
end

-------------------------------------------------------------------------------
--|> EVENTS
-------------------------------------------------------------------------------

--=> FUNCTION: Nameplate Retry
--> Re-checks to see if the target has a nameplate we can attach to...
-->     Sometimes there seems to be a delay between getting a target, and registering the nameplate
local NameplateGetAttempts = 0
local lockPos = false
local nameplate
-------------------------------------------------------
local function NameplateRetry()
	nameplate = C_NamePlate.GetNamePlateForUnit("target")
	if nameplate then
		weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
		CrosshairFadeOutGroup:Stop()
		lockPos = false
	else
		--> Check for valid nameplate existing
		if NameplateGetAttempts < 10 then
			NameplateGetAttempts = NameplateGetAttempts + 1
			C_Timer_After(0.1, NameplateRetry)
		end
	end
end

--=> PLAYER_TARGET_CHANGED
-------------------------------------------------------
local LockedOn = false
local name
local charname
function weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
	LockedOn = false
	--> Activate on enemy
	if (UnitIsEnemy("player", "target") or weizPVP.Options.GLOBAL_DEBUG) and UnitIsPlayer("target") and weizPVP.CoreFrame:IsShown() then
		nameplate = C_NamePlate.GetNamePlateForUnit("target")
		if not nameplate then
			if lockPos == false then
				NameplateGetAttempts = 0
				C_Timer_After(0.1, NameplateRetry)
				lockPos = true
				CrosshairFadeOutGroup:Play()
			end
			return
		end

		lockPos = false
		-- if TargetNameplate ~= nameplate then
		TargetNameplate = nameplate
		weizPVP:GetNameplate(nameplate)
		-- end

		--=> Replace Nameplate elements if enabled
		if weizPVP.Options.Crosshair.NameEnabled then
			NameText:SetAlpha(1)
			NameTextBG:SetAlpha(1)
			NameTextGradBG:Show()
			NameTextGradBGCorner:Show()
			--> NAME TEXT
			name = GetUnitName("target", true)
			charname = gsub(name, "-(.*)", "")
			if strlen(name) > strlen(charname) then
				NameText:SetText(charname .. "|cFFFF00CC*|r")
			else
				NameText:SetText(name)
			end
			NameTextBG:SetText(NameText:GetText())
		else
			NameText:SetAlpha(0)
			NameTextBG:SetAlpha(0)
			NameTextGradBG:Hide()
			NameTextGradBGCorner:Hide()
		end

		if weizPVP.Options.Crosshair.GuildEnabled then
			GuildText:SetAlpha(1)
			GuildTextBG:SetAlpha(1)
			GuildTextGradBG:Show()
			GuildTextGradBGCorner:Show()
			GuildText:SetText(GetGuildInfo("target") or "")
			GuildTextBG:SetText(GuildText:GetText())
			if not GuildText:GetText() or GuildText:GetText() == "" then
				GuildTextGradBG:Hide()
			else
				GuildTextGradBG:Show()
			end
		else
			GuildText:SetAlpha(0)
			GuildTextBG:SetAlpha(0)
			GuildTextGradBG:Hide()
			GuildTextGradBGCorner:Hide()
		end
	else --> Fade out if not an enemy or no target
		CrosshairFadeOutGroup:Play()
		TargetNameplate = nil
	end
end

--=> EVENT: PLAYER_ENTERING_WORLD
-------------------------------------------------------
function weizPVP.CrosshairFrame:PLAYER_ENTERING_WORLD()
	self:PLAYER_TARGET_CHANGED()
end

--=> EVENT: ON_UPDATE
-------------------------------------------------------
local elapsed = GetTime() --> Check time in order to throlle net-o-matic checks to save on recourse.
local prevAlpha = 1
local offsetTime = 0.3

local x1, y1, x2, y2, d1, d2, amountToMove, ratio
local distance, timeLeft, maxRange, inRange

weizPVP.CrosshairFrame:SetScript(
	"OnUpdate",
	function(self)
		if TargetNameplate and weizPVP.Options.Crosshair.Enabled == true then
			--> THROTTLE to every 'offsetTime'

			-------------------------------------------------------
			if GetTime() > (offsetTime + elapsed) then
				-------------------------------------------------------
				--> Get if targetplate exists
				if not TargetNameplate then
					CrosshairFadeOutGroup:Play()
					return
				end
				-------------------------------------------------------
				--> Check if Net-o-Matic can be used on target
				if IsFlying("target") and IsItemInRange("Net-o-Matic 5000", "target") and IsUsableItem("Net-o-Matic 5000") then
					iconNetOMatic:SetAlpha(1)
				else
					iconNetOMatic:SetAlpha(0)
				end

				inRange = nil

				--> Check Range
				_, maxRange = RC:GetRange("target")
				maxRange = tonumber(maxRange) or 60
				if maxRange >= 45 then
					inRange = false
				else
					inRange = true
				end
				RingShine:SetVertexColor(weizPVP:AdjustRGBTint(CurrentClassColor, 0.4))
				CenterRing:SetVertexColor(weizPVP:AdjustRGBTint(CurrentClassColor, 0.2))

				--> If out of range (for most spells/abilties at least)
				if maxRange >= 70 then
					if prevAlpha ~= 0.3 then
						CenterRingFade:SetFromAlpha(prevAlpha)
						CenterRingFade:SetToAlpha(0.3)
						CenterRingFadeGroup:Play()

						RingShineFade:SetFromAlpha(prevAlpha)
						RingShineFade:SetToAlpha(0.3)
						RingShineFadeGroup:Play()
						prevAlpha = 0.3
					end
				elseif not inRange then
					if prevAlpha ~= 0.6 then
						CenterRingFade:SetFromAlpha(prevAlpha)
						CenterRingFade:SetToAlpha(0.6)
						CenterRingFadeGroup:Play()

						RingShineFade:SetFromAlpha(prevAlpha)
						RingShineFade:SetToAlpha(0.6)
						RingShineFadeGroup:Play()
						prevAlpha = 0.6
					end
				else --> Target In Range
					if prevAlpha ~= 1 then
						CenterRingFade:SetFromAlpha(prevAlpha)
						CenterRingFade:SetToAlpha(1)
						CenterRingFadeGroup:Play()

						RingShineFade:SetFromAlpha(prevAlpha)
						RingShineFade:SetToAlpha(1)
						RingShineFadeGroup:Play()
						prevAlpha = 1
					end
				end

				weizPVP.CrosshairFrame:SetFrameLevel(tonumber(TargetNameplate:GetFrameLevel()) - 1)
				weizPVP.CrosshairFrame:SetFrameStrata(TargetNameplate:GetFrameStrata())
				elapsed = GetTime()
			end --> END OF THROTTLED CODE

			--> Run on EVERY frame update
			-------------------------------------------------------
			if Moving and lockPos == false then
				x1, y1 = self:GetCenter()
				x1, y1 = x1 * self:GetEffectiveScale(), y1 * self:GetEffectiveScale()
				x2, y2 = TargetNameplate:GetCenter()
				x2, y2 = x2 * TargetNameplate:GetEffectiveScale(), y2 * TargetNameplate:GetEffectiveScale()
				d1, d2 = y2 - y1, x2 - x1
				distance = (d2 ^ 2 + d1 ^ 2) ^ 0.5
				timeLeft = Moving + 1 - GetTime()
				--> Still moving towards the new/current nameplate from last plate's locattion
				if timeLeft > 0.2 and distance > 4 then
					LockedOn = false
					amountToMove = distance / timeLeft / (weizPVP.Display.CurrentFPS / 8)
					ratio = amountToMove / distance
					x = x1 + ratio * d2
					y = y1 + ratio * d1
					weizPVP.CrosshairFrame:ClearAllPoints()
					weizPVP.CrosshairFrame:SetPoint("CENTER", nil, "BOTTOMLEFT", weizPVP.ScaleCoords(x, y))
				else --> Crosshairs are now anchored to the unit's nameplate
					weizPVP.CrosshairFrame:ClearAllPoints()
					weizPVP.CrosshairFrame:SetPoint("CENTER", nil, "BOTTOMLEFT", weizPVP.ScaleCoords(x2, y2))
					Moving = false
				end
			else
				--> LOCKED ONTO NAMEPLATE; NO MORE CHASING
				if LockedOn == false then
					weizPVP.CrosshairFrame:SetPoint("CENTER", TargetNameplate, "CENTER")
					LockedOn = true
				end
			end
			--> SET LINE END POINTS (monitor edge)
			topLine:SetStartPoint("TOP", weizPVP.CrosshairFrame, 0, UIParent:GetTop() - weizPVP.CrosshairFrame:GetTop())
			leftLine:SetStartPoint("LEFT", weizPVP.CrosshairFrame, UIParent:GetLeft() - weizPVP.CrosshairFrame:GetLeft(), 0)
			rightLine:SetStartPoint("RIGHT", weizPVP.CrosshairFrame, UIParent:GetRight() - weizPVP.CrosshairFrame:GetRight(), 0)
			bottomLine:SetStartPoint("BOTTOM", weizPVP.CrosshairFrame, 0, UIParent:GetBottom() - weizPVP.CrosshairFrame:GetBottom())
		end
	end
)

--=> NAMEPLATE_ADDED
-------------------------------------------------------
local nameplate_added
function weizPVP.CrosshairFrame:NAME_PLATE_UNIT_ADDED(unit)
	if UnitIsUnit("target", unit) then
		nameplate_added = C_NamePlate.GetNamePlateForUnit("target")
		if nameplate_added and UnitIsPlayer("target") and (UnitIsEnemy("player", "target") or weizPVP.Options.GLOBAL_DEBUG) then
			weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
		-- TargetNameplate = nameplate_added
		-- weizPVP:GetNameplate(nameplate_added)
		end
	end
end

--=> NAMEPLATE_REMOVED
-------------------------------------------------------
function weizPVP.CrosshairFrame:NAME_PLATE_UNIT_REMOVED(unit)
	if UnitIsUnit("target", unit) then
		TargetNameplate = nil
		CrosshairFadeOutGroup:Play()
	end
end

--=> CrosshairFrame ON_EVENT
-------------------------------------------------------
weizPVP.CrosshairFrame:SetScript(
	"OnEvent",
	function(self, event, ...)
		return self[event] and self[event](self, ...)
	end
)
